==> syft-0.8.7b16-py2.py3-none-any.whl: syft/types/syft_object_registry.py <==
# stdlib
from collections.abc import Callable
from typing import Any
from typing import TYPE_CHECKING

SYFT_086_PROTOCOL_VERSION = "4"

# third party

# relative
if TYPE_CHECKING:
    # relative
    from .syft_object import SyftObject


class SyftObjectRegistry:
    __object_transform_registry__: dict[str, Callable] = {}
    __object_serialization_registry__: dict[str, dict[int, tuple]] = {}
    __type_to_canonical_name__: dict[type, tuple[str, int]] = {}

    @classmethod
    def register_cls(
        cls, canonical_name: str, version: int, serde_attributes: tuple
    ) -> None:
        if canonical_name not in cls.__object_serialization_registry__:
            cls.__object_serialization_registry__[canonical_name] = {}
        cls.__object_serialization_registry__[canonical_name][version] = (
            serde_attributes
        )

        cls.__type_to_canonical_name__[serde_attributes[7]] = (canonical_name, version)

    @classmethod
    def get_versions(cls, canonical_name: str) -> list[int]:
        available_versions: dict = cls.__object_serialization_registry__.get(
            canonical_name,
            {},
        )
        return list(available_versions.keys())

    @classmethod
    def get_canonical_name_version(cls, obj: Any) -> tuple[str, int]:
        """
        Retrieves the canonical name for both objects and types.

        This function works for both objects and types, returning the canonical name
        as a string. It handles various cases, including built-in types, instances of
        classes, and enum members.

        If the object is not registered in the registry, a ValueError is raised.

        Examples:
            get_canonical_name_version([1,2,3]) -> "list"
            get_canonical_name_version(list) -> "type"
            get_canonical_name_version(MyEnum.A) -> "MyEnum"
            get_canonical_name_version(MyEnum) -> "EnumMeta"

        Args:
            obj: The object or type for which to get the canonical name.

        Returns:
            The canonical name and version of the object or type.
        """

        # NOTE the metaclass of the object is not needed during serde
        # so we can safely ignore it
        if isinstance(obj, type):
            return cls.__type_to_canonical_name__[type]

        obj_type = type(obj)
        if obj_type in cls.__type_to_canonical_name__:
            return cls.__type_to_canonical_name__[obj_type]
        raise ValueError(f"Could not find canonical name for {obj}")

    @classmethod
    def get_serde_properties(cls, canonical_name: str, version: int) -> tuple:
        return cls.__object_serialization_registry__[canonical_name][version]

    @classmethod
    def get_serde_class(cls, canonical_name: str, version: int) -> type["SyftObject"]:
        serde_properties = cls.get_serde_properties(canonical_name, version)
        return serde_properties[7]

    @classmethod
    def get_serde_properties_bw_compatible(
        cls, fqn: str, canonical_name: str, version: int
    ) -> tuple:
        # relative
        from ..serde.recursive import TYPE_BANK

        if canonical_name != "" and canonical_name is not None:
            return cls.get_serde_properties(canonical_name, version)
        else:
            # this is for backward compatibility with 0.8.6
            try:
                # relative
                from ..protocol.data_protocol import get_data_protocol

                serde_props = TYPE_BANK[fqn]
                klass = serde_props[7]
                is_syftobject = hasattr(klass, "__canonical_name__")
                if is_syftobject:
                    canonical_name = klass.__canonical_name__
                    dp = get_data_protocol()
                    try:
                        version_mutations = dp.protocol_history[
                            SYFT_086_PROTOCOL_VERSION
                        ]["object_versions"][canonical_name]
                    except Exception:
                        print(f"could not find {canonical_name} in protocol history")
                        raise

                    version_086 = max(
                        [
                            int(k)
                            for k, v in version_mutations.items()
                            if v["action"] == "add"
                        ]
                    )
                    try:
                        res = cls.get_serde_properties(canonical_name, version_086)

                    except Exception:
                        print(
                            f"could not find {canonical_name} {version_086} in ObjectRegistry"
                        )
                        raise
                    return res
                else:
                    # TODO, add refactoring for non syftobject versions
                    canonical_name = fqn
                    version = 1
                    return cls.get_serde_properties(canonical_name, version)
            except Exception as e:
                print(e)
                raise

    @classmethod
    def has_serde_class(
        cls, fqn: str, canonical_name: str | None, version: int
    ) -> bool:
        # relative
        from ..serde.recursive import TYPE_BANK

        if canonical_name != "" and canonical_name is not None:
            return (
                canonical_name in cls.__object_serialization_registry__
                and version in cls.__object_serialization_registry__[canonical_name]
            )
        else:
            # this is for backward compatibility with 0.8.6
            return fqn in TYPE_BANK

    @classmethod
    def add_transform(
        cls,
        klass_from: str,
        version_from: int,
        klass_to: str,
        version_to: int,
        method: Callable,
    ) -> None:
        mapping_string = f"{klass_from}_{version_from}_x_{klass_to}_{version_to}"
        cls.__object_transform_registry__[mapping_string] = method

    @classmethod
    def get_transform(
        cls, type_from: type["SyftObject"], type_to: type["SyftObject"]
    ) -> Callable:
        # relative
        from .syft_object import SyftBaseObject
        from .syft_object import SyftObject

        for type_from_mro in type_from.mro():
            if issubclass(type_from_mro, SyftObject):
                klass_from = type_from_mro.__canonical_name__
                version_from = type_from_mro.__version__
            else:
                klass_from = type_from_mro.__name__
                version_from = None
            for type_to_mro in type_to.mro():
                if issubclass(type_to_mro, SyftBaseObject):
                    klass_to = type_to_mro.__canonical_name__
                    version_to = type_to_mro.__version__
                else:
                    klass_to = type_to_mro.__name__
                    version_to = None

                mapping_string = (
                    f"{klass_from}_{version_from}_x_{klass_to}_{version_to}"
                )
                if mapping_string in SyftObjectRegistry.__object_transform_registry__:
                    return SyftObjectRegistry.__object_transform_registry__[
                        mapping_string
                    ]
        raise Exception(
            f"No mapping found for: {type_from} to {type_to} in"
            f"the registry: {SyftObjectRegistry.__object_transform_registry__.keys()}"
        )

==> syft-0.8.8-py2.py3-none-any.whl: syft/types/syft_object_registry.py <==
# stdlib
from collections.abc import Callable
from typing import Any
from typing import TYPE_CHECKING

SYFT_086_PROTOCOL_VERSION = "4"

# third party

# relative
if TYPE_CHECKING:
    # relative
    from .syft_object import SyftObject


class SyftObjectRegistry:
    __object_transform_registry__: dict[str, Callable] = {}
    __object_serialization_registry__: dict[str, dict[int, tuple]] = {}
    __type_to_canonical_name__: dict[type, tuple[str, int]] = {}

    @classmethod
    def register_cls(
        cls, canonical_name: str, version: int, serde_attributes: tuple
    ) -> None:
        if canonical_name not in cls.__object_serialization_registry__:
            cls.__object_serialization_registry__[canonical_name] = {}
        cls.__object_serialization_registry__[canonical_name][version] = (
            serde_attributes
        )

        cls.__type_to_canonical_name__[serde_attributes[7]] = (canonical_name, version)

    @classmethod
    def get_versions(cls, canonical_name: str) -> list[int]:
        available_versions: dict = cls.__object_serialization_registry__.get(
            canonical_name,
            {},
        )
        return list(available_versions.keys())

    @classmethod
    def get_identifier_for_type(cls, obj: Any) -> tuple[str, int]:
        """
        This is to create the string in nonrecursiveBlob
        """
        return cls.__type_to_canonical_name__[obj]

    @classmethod
    def get_canonical_name_version(cls, obj: Any) -> tuple[str, int]:
        """
        Retrieves the canonical name for both objects and types.

        This function works for both objects and types, returning the canonical name
        as a string. It handles various cases, including built-in types, instances of
        classes, and enum members.

        If the object is not registered in the registry, a ValueError is raised.

        Examples:
            get_canonical_name_version([1,2,3]) -> "list"
            get_canonical_name_version(list) -> "type"
            get_canonical_name_version(MyEnum.A) -> "MyEnum"
            get_canonical_name_version(MyEnum) -> "type"

        Args:
            obj: The object or type for which to get the canonical name.

        Returns:
            The canonical name and version of the object or type.
        """

        # for types we return "type"
        if isinstance(obj, type):
            return cls.__type_to_canonical_name__[type]

        obj_type = type(obj)
        if obj_type in cls.__type_to_canonical_name__:
            return cls.__type_to_canonical_name__[obj_type]
        raise ValueError(f"Could not find canonical name for {obj}")

    @classmethod
    def get_serde_properties(cls, canonical_name: str, version: int) -> tuple:
        try:
            return cls.__object_serialization_registry__[canonical_name][version]
        except Exception:
            # This is a hack for python 3.10 in which Any is not a type
            # if the server uses py>3.10 and the client 3.10 this goes wrong
            if canonical_name == "Any_typing._SpecialForm":
                return cls.__object_serialization_registry__["Any"][version]
            else:
                raise

    @classmethod
    def get_serde_class(cls, canonical_name: str, version: int) -> type["SyftObject"]:
        serde_properties = cls.get_serde_properties(canonical_name, version)
        return serde_properties[7]

    @classmethod
    def has_serde_class(cls, canonical_name: str | None, version: int) -> bool:
        # relative
        return (
            canonical_name in cls.__object_serialization_registry__
            and version in cls.__object_serialization_registry__[canonical_name]
        )

    @classmethod
    def add_transform(
        cls,
        klass_from: str,
        version_from: int,
        klass_to: str,
        version_to: int,
        method: Callable,
    ) -> None:
        mapping_string = f"{klass_from}_{version_from}_x_{klass_to}_{version_to}"
        cls.__object_transform_registry__[mapping_string] = method

    @classmethod
    def get_transform(
        cls, type_from: type["SyftObject"], type_to: type["SyftObject"]
    ) -> Callable:
        # relative
        from .syft_object import SyftBaseObject
        from .syft_object import SyftObject

        for type_from_mro in type_from.mro():
            if issubclass(type_from_mro, SyftObject):
                klass_from = type_from_mro.__canonical_name__
                version_from = type_from_mro.__version__
            else:
                klass_from = type_from_mro.__name__
                version_from = None
            for type_to_mro in type_to.mro():
                if issubclass(type_to_mro, SyftBaseObject):
                    klass_to = type_to_mro.__canonical_name__
                    version_to = type_to_mro.__version__
                else:
                    klass_to = type_to_mro.__name__
                    version_to = None

                mapping_string = (
                    f"{klass_from}_{version_from}_x_{klass_to}_{version_to}"
                )
                if mapping_string in SyftObjectRegistry.__object_transform_registry__:
                    return SyftObjectRegistry.__object_transform_registry__[
                        mapping_string
                    ]
        raise Exception(
            f"No mapping found for: {type_from} to {type_to} in"
            f"the registry: {SyftObjectRegistry.__object_transform_registry__.keys()}"
        )
